/*!
* \file error.hpp
* \brief Error messages and execution ending handlers
 */

#pragma once

#include "AstralbrewEngine/Debug/symbols.hpp"

/*! \cond HIDDEN_SYMBOLS */ 
#define LOCALIZED_ERROR_MSG(message) \
	"Fatal error\n\n" message "\n\n" __FILE__ "\nLine " STRINGIZE(__LINE__)
	
#define LOCALIZED_ERROR(message) \
	Astralbrew::Debug::fatal_error(LOCALIZED_ERROR_MSG(message))
	
/*! \endcond */ 

/*!
* \brief Produces a fatal error and displays the given message
* 
* \param message Error message
 */
#define FATAL_ERROR(message) \
	Astralbrew::Debug::fatal_error(message)

namespace Astralbrew::Debug
{
	/*!
	* \brief Produces a fatal error and displays the given message
	* 
	* \param [in] message Error message
	 */
	void fatal_error(const char* message);		
}