/*!
* \file colors.hpp
* \brief Colors definitions to improve readability
 */

#pragma once

#include <gba.h>
#include "AstralbrewEngine/types/templates.hpp"

namespace Astralbrew::Drawing::Colors
{
	extern const u16 Black; /*!< Black color*/
	extern const u16 DarkBlue; /*!< Dark Blue color*/
	extern const u16 DarkGreen; /*!< Dark Green color*/
	extern const u16 DarkCyan; /*!< Dark Cyan color*/
	extern const u16 DarkRed; /*!< Dark Red color*/
	extern const u16 DarkMagenta; /*!< Dark Magenta color*/
	extern const u16 DarkYellow; /*!< Dark Yellow color*/
	extern const u16 DarkGray; /*!< Dark Gray color*/
	extern const u16 Blue; /*!< Blue color*/
	extern const u16 Green; /*!< Green color*/
	extern const u16 Cyan; /*!< Cyan color*/
	extern const u16 Red; /*!< Red color*/
	extern const u16 Magenta; /*!< Magenta color*/
	extern const u16 Yellow; /*!< Yellow color*/
	extern const u16 Gray; /*!< Gray color*/
	extern const u16 White; /*!< White color*/

	extern const u16 SYS_COLORS[]; /*!< The 16 system console colors */
	
	void set_bg_palette_system_colors(int palette_bank);
	
	constexpr int brighten(int color, int factor) // not working!
	{
		int r = color & 0x1f; color>>=5;
		int g = color & 0x1f; color>>=5;
		int b = color & 0x1f; 
		int cmax = max(r,g,b);
		int cmin = min(r,g,b);
		int d = cmax-cmin;
			
		int h = 0;
		int s = 0;
		int l = (cmax+cmin)/2;		
		
		if(d!=0)
		{
			if(cmax==r)
			{
				h=60*(((g-b)/d)%6);			
			}
			else if(cmax==g)
			{
				h=60*(((b-r)/d)+2);			
			}
			else
			{
				h=60*(((r-g)/d)+2);			
			}			
			s = 2*l-1;
			s = d / (31 - (s>=0 ? s : -s));
		}
		l += (31-l)*factor/16;
		
		int c = 2*l-1;
		c = (31-((c>0)?c:-c))*s;
		int x = c*(31 - ((h/60)%2-1));
		int m = l-c/2;
		
		if(0<=h && h<60)  r=c, g=x, b=0; 
		else if(60<=h && h<120) r=x, g=c, b=0; 
		else if(120<=h && h<180) r=0, g=c, b=x; 
		else if(180<=h && h<240) r=0, g=x, b=c;		
		else if(240<=h && h<300) r=x, g=0, b=c;		
		else r=c, g=0, b=x;		
		r+=m, g+=m, b+=m;
		return r|(g<<5)|(b<<10);	
	}
}