/*!
* \file hitbox.hpp
* \brief Hitbox definitions
 */

#pragma once

#include <gba.h>

#include "AstralbrewEngine/objects/oam.hpp"

namespace Astralbrew::Entity
{
	/*! \brief Rectangular hitbox for 2d entities
	 */
	class Hitbox
	{
	public:
		u8 left, top, width, height;
		
		/*! \brief creates an empty hitbox*/
		Hitbox();
		
		/*! \brief creates a Hitbox instance 
			\param left    hitbox left coordinate
			\param top     hitbox top coordinate
			\param width   hitbox width
			\param height  hitbox height
		 */
		Hitbox(u8 left, u8 top, u8 width, u8 height);
		
		/*! \brief creates hitbox component that fits an object size type
			\param size size of an object that may correspond to the hitbox
		 */
		Hitbox(Astralbrew::Objects::ObjSize size);
	};
}