#pragma once

namespace Astralbrew::SaveTool
{
	struct SaveBuffer
	{
		int* offset;
		int length;
	};
	
	class SaveFileBody { };
	
	class SaveFile
	{	
	private:		
		const char* type;
		static void st_write(const void* source, int size, int offset);
		static void st_read(void* destination, int size, int offset);
	protected:			
		int size;
		int body_start_offset = 0;
		
		template<typename T> 
		T read(int offset) const
		{
			T result;
			st_read(&result, sizeof(T), offset);
			return result;
		}
		
		template<typename T>
		void write(int offset, const T& value) const
		{
			st_write(&value, sizeof(T), offset);
		}
		
	public:			
		SaveFile();
		virtual ~SaveFile() = default;
	};
	
	template<typename Data> class StructuredSaveFile : public SaveFile
	{
	private:
		Data _data;
	public:
		void load() { _data = read<Data>(body_start_offset); }		
		void save() { write<Data>(body_start_offset, _data); }
		
		Data& data() { return _data; };
		
		~StructuredSaveFile() = default;
	};
	
			
}

