#pragma once

#include <gba.h>

namespace Astralbrew::Text
{
	enum class ExtendedFontChart
	{		
		None  = 0, 
		ExtL1 = 1<<0,
		ExtL2 = 1<<1,
		ExtCy = 1<<2
	};
		
	enum class FontSize
	{
		FONT_8x8   = 1,			
		FONT_8x16  = 2,
		FONT_16x16 = 4
	};
	
	class ReadOnlyFont final
	{
	private:
		// this serves as astralfont file header
		// glyph data comes after
		short char_size;
		short charts;
	public:		
		ReadOnlyFont() = delete; 		
		~ReadOnlyFont() = delete;
		
		int char_width() const;
		int char_height() const;
		int get_header_size() const;
		int get_chart_size() const;
		const void* get_ascii_base_addr() const;
		const void* get_ascii_extended_addr(ExtendedFontChart ext_chart) const;
		
		int get_glyph_width(int code, ExtendedFontChart ext_chart = ExtendedFontChart::None) const;
						
		const void* get_glyph(int code, ExtendedFontChart ext_chart = ExtendedFontChart::None) const;	

		FontSize font_size() const;
	};	
}

namespace Astralbrew::Resources::Fonts
{
	extern const Astralbrew::Text::ReadOnlyFont default_8x8;
    extern const Astralbrew::Text::ReadOnlyFont default_8x16;    
}