#pragma once

#include "AstralbrewEngine/drawing/brush.hpp"
#include "AstralbrewEngine/video/background.hpp"
#include "AstralbrewEngine/text/font.hpp"

namespace Astralbrew::Text
{
	class VwfEngine
	{
	private:
		const ReadOnlyFont& font;		
		short char_width = 0;
		short char_height = 0;
		ExtendedFontChart ext_chart = ExtendedFontChart::None;
		
		
		unsigned short* render_area = nullptr;
		short render_rows_count;
		short render_columns_count;
		int crt_row = 0, crt_col = 0;
		int px_offset = 0;				
		
	public:
		VwfEngine(const ReadOnlyFont& font, ExtendedFontChart ext_chart = ExtendedFontChart::None);
		void set_render_space(void* buffer, int rows_count, int columns_count);
		
		int put_char(char code, Astralbrew::Video::BgPaletteType pal_type, const Astralbrew::Drawing::Brush& text_brush);
		int put_word(const char* word, Astralbrew::Video::BgPaletteType pal_type, const Astralbrew::Drawing::Brush& text_brush);
		int put_text(const char* text, Astralbrew::Video::BgPaletteType pal_type, const Astralbrew::Drawing::Brush& text_brush);
		
		void clear(Astralbrew::Video::BgPaletteType pal_type);
		
		
		inline const ReadOnlyFont& get_font() const { return font; }
		
		static void prepare_map(const VwfEngine& vwf, u16* map_ptr, int map_stride, int map_x, int map_y, int palette_bank = 0);
	};	
}

