/*!
* \file memory.hpp
* \brief Memory targetted definitions
 */

#pragma once

#include <gba.h>

namespace Astralbrew
{
	namespace Utils
	{		
		/*! \brief constant 0
			Useful when a constant number should be passed by address
		*/
		extern const u32 ZERO;
		
		/*! \brief fills a number of words with 0 at a given offset
			\param offset address of then first byte to set
			\param nbytes number of bytes to zeroize (GBA BIOS requires this number to be a multiple of 32 - 8 words)
		 */			
		void zeroize(void* offset, int nbytes);
		
		/*! \brief fills a number of words with all bits of 1 at a given offset
			\param offset address of then first byte to set
			\param nwords number of words to fill with bits of 1 (GBA BIOS requires this number to be a multiple of 8)
		 */
		void setbits(void* offset, u32 nwords);
		
		/*! \brief copies memory from one address to another
			\param source address to copy bytes form
			\param destination address to copy bytes to
			\param size number of bytes to copy 
		 */
		void copy(const void* source, void* destination, u32 size);
	}
}