#pragma once

#include <gba.h>

namespace Astralbrew::Video
{
	// adding bg handlers because libgba doesn't have them
	enum BgSize
	{
		Text256x256 = 0,
		Text512x256 = 1,
		Text256x512 = 2,
		Text512x512 = 3,
		RotS128x128 = 0,
		RotS256x256 = 1,
		RotS512x512 = 2,
		RotS1024x1024 = 3
	};
	
	enum BgPaletteType
	{
		Pal4bit = 0,
		Pal8bit = 1
	};
	
	struct BgState
	{
		int scrollX;
		int scrollY;
		int centerX;
		int centerY;
		int angle;
		int scaleX;
		int scaleY;		
		bool request_update;
	};
	
	extern __attribute__((section(".ewram.bgState"))) BgState bgState[4];
	
	void bgInit(int id, BgSize size, BgPaletteType palette, int char_base, int screen_base);
	
	bool bgIsText(int id);
	
	void bgWrap(int id);
	void bgUnwrap(int id);
	
	void bgMosaicEnable(int id);
	void bgMosaicDisable(int id);
	
	void bgScroll(int id, int dx, int dy);
	void bgSetScroll(int id, int x, int y);
	
	void bgScrollf(int id, int dx, int dy);
	void bgSetScrollf(int id, int x, int y);
	
	void bgSetCenter(int id, int x,int y);
	void bgSetCenterf(int id, int x,int y);
	
	void bgSetRotate(int id, int angle);
	void bgRotate(int id, int angle);
	
	void bgSetScale(int id, int sx, int sy);
	
	void bgSetPriority(int id, int priority);
	
	void bgUpdate();
	
	void bgShow(int id);
	void bgHide(int id);
	bool bgIsVisible(int id);
	
	u16* bgGetMapPtr(int id);
	u16* bgGetTilesPtr(int id);
	
	
	void bgSetAlpha(int id1, int a1, int id2, int a2);
	
	void copyTiles256(const void* src, void* dest, int len, const void* pal, int pal_len, int pal_offset);
	
	int bgBlockOwnerMap(const void* const address);	
	int bgTileId(const void* address, int ref_map_id);
	
	#define CHAR_BASE_OFFSET(id, offset) ((u16*)(((int)CHAR_BASE_ADR(id))+((offset+1)/2)*2))
	#define BG_PALETTE_OFFSET(offset) ((u16*)(((int)BG_PALETTE)+2*offset))
	
	#define BG_PALETTE_BANK(i) ((u16*)(((int)BG_PALETTE)+32*i))
	
	#define tiles_size_4bpp(x) ((x)*32)
	#define tiles_size_8bpp(x) ((x)*64)
}


