/*!
* \file registers.hpp
* \brief Wrap around video registers
 */

#pragma once

namespace Astralbrew::Video
{
	/*!
	 * \brief Set video mode
	 * \param mode 0..5	 
	 */
	void setMode(int mode);
	
	/*!
	 * \brief Get video mode
	 * \return video mode 0..5	 
	 */
	int getMode();
	
	/*!
	 * \brief Enable green swap state
	 */
	void greenSwapEnable();

	/*!
	 * \brief Disable green swap state
	 */
	void greenSwapDisable();

	/*!
	 * \brief Change green swap state
	 */
	void greenSwapSwitch();

	/*!
	 * \brief Enable 1D sprites	
	 */
	void objEnable1D();
	
	/*!
	 * \brief Enable 2D sprites 
	 */
	void objEnable2D();
	
	/*!
	 * \brief Disable sprites
	 */
	void objDisable();
	
	/*!
	* \brief Check if 1D sprites are enabled
	* \return true if 1D sprites are enabled, false otherwise
	 */
	bool objIs1D();
	
	/*!
	* \brief Check if 2D sprites are enabled
	* \return true if 2D sprites are enabled, false otherwise
	 */
	bool objIs2D();
	
	/*!
	* \brief Check if sprites are enabled
	* \return true if sprites are enabled, false otherwise
	 */
	bool objIsEnabled();
}