/*!
* \file camera.hpp
* \brief 2D world camera definition
 */

#pragma once

#include "AstralbrewEngine/entity/localizable.hpp"
#include "AstralbrewEngine/types/sf24.hpp"

namespace Astralbrew::World
{
	/*! \brief Camera position teller object for 2D worlds
	 */
	class Camera 
	{		
	private:
		sf24 x, y;
		Entity::Localizable* target;
		s16 bndw=32767, bndh=32767;
	public:
		Camera();
		/*!
		* \brief Creates camera positioned at specified coordinated		
		* \param [in] x Camera x
		* \param [in] y Camera y
		 */
		Camera(sf24 x, sf24 y);
		
		/*!
		* \brief Gets camera x coordinate		
		* \return Camera x coordinate			
		* \details Camera coordinates depend on the followed entity, is specified
		 */
		s16 get_x() const;
		
		/*!
		* \brief Gets camera y coordinate		
		* \return Camera y coordinate			
		* \details Camera coordinates depend on the followed entity, is specified
		 */
		s16 get_y() const;
		
		/*!
		* \brief Sets camera x coordinate		
		* \param [in] value new camera x coordinate			
		* \details Setting coordinates detaches the camera from its followed entity
		 */
		void set_x(const sf24& value);	
		
		/*!
		* \brief Sets camera y coordinate		
		* \param [in] value new camera y coordinate			
		* \details Setting coordinates detaches the camera from its followed entity
		 */
		void set_y(const sf24& value);
		
		/*!
		* \brief Set camera exploration limits
		* 
		* \param [in] w Width
		* \param [in] h Height				
		 */
		void set_bounds(s16 w, s16 h);
		
		/*!
		* \brief Moves camera relative to its initial coordinates
		* 
		* \param [in] x Quantity to move along the x axis
		* \param [in] y Quantity to move along the y axis		
	
		* \details If camera bounds are reached, the position sticks to them.
		 */
		void move(sf24 x, sf24 y);
		
		/*!
		* \brief always focuses the camera on the specified entity
		* 
		* \param target entity				
		* \details The camera will follow the entity's position
		 */
		void follow(Entity::Localizable* entity);		
	};
}